/**
 * SPDX-FileCopyrightText: (c) 2000 Liferay, Inc. https://liferay.com
 * SPDX-License-Identifier: LGPL-2.1-or-later OR LicenseRef-Liferay-DXP-EULA-2.0.0-2023-06
 */

import pathToRegexp from 'path-to-regexp';

import {capitalize} from '../../../util/util.es';
import {parse, stringify} from '../../router/queryString.es';

const asFilterObject = (items, key, name, pinned) => ({
	items,
	key,
	name,
	pinned,
});

const buildFilterItem = (data) => {
	if (typeof data === 'string') {
		return {
			active: true,
			key: data,
		};
	}

	return {
		...data,
		active: true,
	};
};

const buildFilterItems = ({
	formatItem,
	items,
	propertyKey = 'key',
	selectedKeys,
}) => {
	return items.map((item, index) => {
		const key = String(item[propertyKey]);

		if (formatItem) {
			item = formatItem(item);
		}

		return {
			...item,
			active: selectedKeys && selectedKeys.includes(key),
			dividerAfter: item.dividerAfter && !!items[index + 1],
			key,
		};
	});
};

const getFilterKeys = (items = []) => items.map(({key}) => key);

const getFiltersParam = (queryString) => {
	const queryParams = parse(queryString);

	return queryParams.filters || {};
};

const getCapitalizedFilterKey = (prefix, key) => {
	return prefix ? `${prefix}${capitalize(key)}` : key;
};

const getFilterResults = (prefixedKeys, pinnedValues, titles, values) => {
	const filterResults = [];

	prefixedKeys.forEach((prefixedKey, index) => {
		if (values[prefixedKey]) {
			filterResults.push(
				asFilterObject(
					values[prefixedKey],
					prefixedKey,
					titles[index],
					pinnedValues[index]
				)
			);
		}
	});

	return filterResults;
};

const getFilterValues = (filterState) => {
	const filterValues = {};

	Object.keys(filterState).forEach((key) => {
		if (filterState[key]) {
			filterValues[key] = getFilterKeys(filterState[key]);
		}
	});

	return filterValues;
};

const getSelectedItemsQuery = (items, key, queryString) => {
	const queryParams = parse(queryString);

	const filtersParam = queryParams.filters || {};

	queryParams.filters = {
		...filtersParam,
		[key]: items.filter((item) => item.active).map((item) => item.key),
	};

	return stringify(queryParams);
};

const getSelectedItems = (filterResults) => {
	return filterResults.filter((filter) => {
		filter.items = filter.items
			? filter.items.filter((item) => item.active)
			: [];

		return !!filter.items.length;
	});
};

const mergeItemsArray = (baseItems = [], ...items) => {
	items = items.filter((value) => value !== undefined && value !== null);

	return baseItems.concat(...items);
};

const reduceFilters = (filterItems, paramKey) => {
	return filterItems.reduce(
		(acc, cur) => `&${paramKey}=${cur.key}${acc}`,
		''
	);
};

const removeFilters = (queryString) => {
	const queryParams = parse(queryString);

	queryParams.filters = null;
	queryParams.search = null;

	return stringify(queryParams);
};

const removeItem = (filterKey, itemToRemove, queryString) => {
	const queryParams = parse(queryString);

	const filtersParam = queryParams.filters || {};

	const filterValues = filtersParam[filterKey] || [];

	filtersParam[filterKey] = filterValues.filter(
		(filterValue) => filterValue !== itemToRemove.key
	);

	queryParams.filters = filtersParam;

	return stringify(queryParams);
};

const replaceHistory = (filterQuery, routerProps) => {
	const {
		history,
		location: {search},
		match: {params, path},
	} = routerProps;

	const pathname = pathToRegexp.compile(path)({...params, page: 1});

	if (filterQuery !== search) {
		history.replace({
			pathname,
			search: filterQuery,
		});
	}
};

export {
	asFilterObject,
	buildFilterItem,
	buildFilterItems,
	getCapitalizedFilterKey,
	getFilterKeys,
	getFiltersParam,
	getFilterResults,
	getFilterValues,
	getSelectedItems,
	getSelectedItemsQuery,
	mergeItemsArray,
	reduceFilters,
	removeFilters,
	removeItem,
	replaceHistory,
};
